@@ -16,16 +16,39 @@
 import json
 import logging
 import os
+import shlex
 import sys
 
 from subiquitycore.controller import BaseController
-from subiquitycore.models import IdentityModel
 from subiquitycore.utils import disable_console_conf, run_command
 
 from console_conf.ui.views import IdentityView, LoginView
 
 log = logging.getLogger('console_conf.controllers.identity')
 
+
+def get_core_version():
+    """ For a ubuntu-core system, return its version or None """
+
+    path = "/usr/lib/os-release"
+    try:
+        with open(path, "r") as fp:
+            content = fp.read()
+    except FileNotFoundError:
+        return None
+
+    version = None
+    for line in shlex.split(content):
+        key, _, value = line.partition("=")
+        if key == "ID" and value != "ubuntu-core":
+            return None
+        if key == "VERSION_ID":
+            version = value
+            break
+
+    return version
+
+
 def get_device_owner():
     """ Check if device is owned """
 
@@ -46,26 +69,27 @@ def get_device_owner():
             return result
     return None
 
+
 def host_key_fingerprints():
     """Query sshd to find the host keys and then fingerprint them.
 
     Returns a sequence of (key-type, fingerprint) pairs.
     """
     config = run_command(['sshd', '-T'])
-    if config['status'] != 0:
+    if config.returncode != 0:
         log.debug("sshd -T failed %r", config['err'])
         return []
     keyfiles = []
-    for line in config['output'].splitlines():
+    for line in config.stdout.splitlines():
         if line.startswith('hostkey '):
             keyfiles.append(line.split(None, 1)[1])
     info = []
     for keyfile in keyfiles:
-        result = run_command(['ssh-keygen', '-lf', keyfile])
-        if result['status'] != 0:
-            log.debug("ssh-keygen -lf %s failed %r", keyfile, result['err'])
+        cp = run_command(['ssh-keygen', '-lf', keyfile])
+        if cp.returncode != 0:
+            log.debug("ssh-keygen -lf %s failed %r", keyfile, cp.stderr)
             continue
-        parts = result['output'].strip().split()
+        parts = cp.stdout.strip().split()
         length, fingerprint, host, keytype = parts
         keytype = keytype.strip('()')
         info.append((keytype, fingerprint))
@@ -89,25 +113,32 @@ The {keytype} host key fingerprints is:
 
 def host_key_info():
     fingerprints = host_key_fingerprints()
+    if len(fingerprints) == 0:
+        return []
     if len(fingerprints) == 1:
         [(keytype, fingerprint)] = fingerprints
-        return single_host_key_tmpl.format(keytype=keytype, fingerprint=fingerprint)
+        return single_host_key_tmpl.format(keytype=keytype,
+                                           fingerprint=fingerprint)
     lines = [host_keys_intro]
     longest_type = max([len(keytype) for keytype, _ in fingerprints])
     for keytype, fingerprint in fingerprints:
-        lines.append(host_key_tmpl.format(keytype=keytype, fingerprint=fingerprint, width=longest_type))
+        lines.append(host_key_tmpl.format(keytype=keytype,
+                                          fingerprint=fingerprint,
+                                          width=longest_type))
     return "".join(lines)
 
+
 login_details_tmpl = """\
-Ubuntu Core 16 on {first_ip} ({tty_name})
+Ubuntu Core {version} on {first_ip} ({tty_name})
 {host_key_info}
 To login:
 {sshcommands}
 Personalize your account at https://login.ubuntu.com.
 """
 
+
 login_details_tmpl_no_ip = """\
-Ubuntu Core 16 on <no ip address> ({tty_name})
+Ubuntu Core {version} on <no ip address> ({tty_name})
 
 You cannot log in until the system has an IP address. (Is there
 supposed to be a DHCP server running on your network?)
@@ -115,18 +146,24 @@ supposed to be a DHCP server running on your network?)
 Personalize your account at https://login.ubuntu.com.
 """
 
+
 def write_login_details(fp, username, ips):
     sshcommands = "\n"
     for ip in ips:
-        sshcommands += "    ssh %s@%s\n"%(username, ip)
-    tty_name = os.ttyname(0)[5:] # strip off the /dev/
+        sshcommands += "    ssh %s@%s\n" % (username, ip)
+    tty_name = os.ttyname(0)[5:]  # strip off the /dev/
+    version = get_core_version() or "16"
     if len(ips) == 0:
         fp.write(login_details_tmpl_no_ip.format(
-            sshcommands=sshcommands, tty_name=tty_name))
+            sshcommands=sshcommands, tty_name=tty_name, version=version))
     else:
         first_ip = ips[0]
-        fp.write(login_details_tmpl.format(
-            sshcommands=sshcommands, host_key_info=host_key_info(), tty_name=tty_name, first_ip=first_ip))
+        fp.write(login_details_tmpl.format(sshcommands=sshcommands,
+                                           host_key_info=host_key_info(),
+                                           tty_name=tty_name,
+                                           first_ip=first_ip,
+                                           version=version))
+
 
 def write_login_details_standalone():
     owner = get_device_owner()
@@ -134,7 +171,8 @@ def write_login_details_standalone():
         print("No device owner details found.")
         return 0
     from probert import network
-    from subiquitycore.models.network import NETDEV_IGNORED_IFACE_NAMES, NETDEV_IGNORED_IFACE_TYPES
+    from subiquitycore.models.network import (NETDEV_IGNORED_IFACE_NAMES,
+                                              NETDEV_IGNORED_IFACE_TYPES)
     import operator
     observer = network.UdevObserver()
     observer.start()
@@ -149,10 +187,9 @@ def write_login_details_standalone():
                 ips.append(addr.ip)
     if len(ips) == 0:
         tty_name = os.ttyname(0)[5:]
-        print("Ubuntu Core 16 on <no ip address> ({})".format(tty_name))
-        print()
-        print("You cannot log in until the system has an IP address.")
-        print("(Is there supposed to be a DHCP server running on your network?)")
+        version = get_core_version() or "16"
+        print(login_details_tmpl_no_ip.format(tty_name=tty_name,
+                                              version=version))
         return 2
     write_login_details(sys.stdout, owner['username'], ips)
     return 0
@@ -162,20 +199,23 @@ class IdentityController(BaseController):
 
     def __init__(self, common):
         super().__init__(common)
-        self.model = IdentityModel(self.opts)
+        self.model = self.base_model.identity
 
     def default(self):
         title = "Profile setup"
         excerpt = "Enter an email address from your account in the store."
         footer = ""
         self.ui.set_header(title, excerpt)
-        self.ui.set_footer(footer, 40)
+        self.ui.set_footer(footer)
         self.ui.set_body(IdentityView(self.model, self, self.opts, self.loop))
         device_owner = get_device_owner()
         if device_owner is not None:
             self.model.add_user(device_owner)
-            key_file = os.path.join(device_owner['homedir'], ".ssh/authorized_keys")
-            self.model.user.fingerprints = run_command(['ssh-keygen', '-lf', key_file])['output'].replace('\r', '').splitlines()
+            key_file = os.path.join(device_owner['homedir'],
+                                    ".ssh/authorized_keys")
+            self.model.user.fingerprints = (
+                run_command(['ssh-keygen', '-lf',
+                             key_file]).stdout.replace('\r', '').splitlines())
             self.login()
 
     def identity_done(self, email):
@@ -189,13 +229,15 @@ class IdentityController(BaseController):
         else:
             self.ui.frame.body.progress.set_text("Contacting store...")
             self.loop.draw_screen()
-            result = run_command(["snap", "create-user", "--sudoer", "--json", email])
+            cp = run_command(
+                ["snap", "create-user", "--sudoer", "--json", email])
             self.ui.frame.body.progress.set_text("")
-            if result['status'] != 0:
-                self.ui.frame.body.error.set_text("Creating user failed:\n" + result['err'])
+            if cp.returncode != 0:
+                self.ui.frame.body.error.set_text(
+                    "Creating user failed:\n" + cp.stderr)
                 return
             else:
-                data = json.loads(result['output'])
+                data = json.loads(cp.stdout)
                 result = {
                     'realname': email,
                     'username': data['username'],
@@ -230,7 +272,8 @@ class IdentityController(BaseController):
 
     def login_done(self):
         if not self.opts.dry_run:
-            # stop the console-conf services (this will kill the current process).
+            # stop the console-conf services (this will kill the
+            # current process).
             disable_console_conf()
 
         self.signal.emit_signal('quit')
